<?php

class ControllerPaymentBetatransfer extends Controller
{
    private static $baseURL = 'https://betatransfer.io/api';

    public function __construct($registry)
    {
        parent::__construct($registry);
        $this->load->language('payment/betatransfer');
    }

    public function index()
    {
        $this->load->model('checkout/order');
        $order_info = $this->model_checkout_order->getOrder($this->session->data['order_id']);

        $error = null;
        $systems = $this->getPaymentSystems($order_info['currency_code']);

        if (empty($systems) || !$this->is_valid_payment_system($systems)) {
            $error = $this->language->get('text_error_system');
        }

        $data = array();
        $data['error'] = null;

        if (!$error) {
            $params = array_filter([
                'amount' => $this->currency->format($order_info['total'], $order_info['currency_code'], $order_info['currency_value'], false),
                'currency' => $order_info['currency_code'],
                'orderId' => $this->session->data['order_id'],
                'paymentSystem' => $this->config->get('betatransfer_payment_system'),
                'lang' => $this->language->get('code'),
                'urlSuccess' => HTTPS_SERVER . 'index.php?route=payment/betatransfer/success',
                'urlFail' => HTTPS_SERVER . 'index.php?route=payment/betatransfer/fail',
                'urlResult' => HTTPS_SERVER . 'index.php?route=payment/betatransfer/callback',
                'payerId' => hash('crc32', $order_info['customer_id']),
                'payerPhone' => !empty($order_info['telephone']) ? $order_info['telephone'] : null,
                'payerEmail' => !empty($order_info['email']) ? $order_info['email'] : null,
                'payer_firstname' => !empty($order_info['firstname']) ? $order_info['firstname'] : null,
                'payer_lastname' => !empty($order_info['lastname']) ? $order_info['lastname'] : null,
            ]);

            $params['sign'] = $this->generateSign($params);
            $response = $this->sendRequest($params);

            if (empty($response['urlPayment'])) {
                $error = $this->language->get('text_error_response');
            }

            if (!$error) {
                $url = parse_url($response['urlPayment']);
                $action = $url['scheme'] . '://' . $url['host'];

                if (!empty($url['path'])) {
                    $action = $action . $url['path'];
                }

                $parameters = array();

                if (!empty($url['query'])) {
                    parse_str($url['query'], $parameters);
                }

                $data['button_confirm'] = $this->language->get('button_confirm');
                $data['method'] = 'GET';
                $data['action'] = $action;
                $data['parameters'] = $parameters;
            }
        }

        if ($error) {
            $data['error'] = $error;
        }

        return $this->load->view('default/template/payment/betatransfer.tpl', $data);
    }

    public function callback()
    {
        if ($this->request->server['REQUEST_METHOD'] == 'POST') {
            $bt_response = $this->request->post;
            $order_id = (int) $bt_response['orderId'];

            $this->load->model('checkout/order');
            $order = $this->model_checkout_order->getOrder($order_id);

            if (!$order) {
                $this->sendForbidden(sprintf($this->language->get('text_error_order_not_found'), $order_id));
                return false;
            }

            $this->model_checkout_order->addOrderHistory($order_id,
                $this->config->get('betatransfer_order_status_success'),
                sprintf($this->language->get('text_comment'), $bt_response['paymentSystem'], $bt_response['orderAmount']),
                true);

            echo 'OK';
            header("HTTP/1.1 200 OK");
        }
    }

    public function success()
    {
        if ($this->session->data['payment_method']['code'] == 'betatransfer') {
            $this->load->model('checkout/order');
            $this->model_checkout_order->addOrderHistory($this->session->data['order_id'], $this->config->get('betatransfer_order_status_success'), '', true);
        }

        $this->response->redirect($this->url->link('checkout/success', '', 'SSL'));
    }

    public function fail()
    {
        if ($this->session->data['payment_method']['code'] == 'betatransfer') {
            $this->load->model('checkout/order');
            $this->model_checkout_order->addOrderHistory($this->session->data['order_id'], $this->config->get('betatransfer_order_status_fail'), '', true);
        }

        $this->response->redirect($this->url->link('checkout/failure', '', 'SSL'));
    }

    public function confirm()
    {
        if ($this->session->data['payment_method']['code'] == 'betatransfer') {
            $this->load->model('checkout/order');
            $this->model_checkout_order->addOrderHistory($this->session->data['order_id'], $this->config->get('betatransfer_order_status_confirm'), '', true);
        }
    }

    protected function getPaymentSystems($currency)
    {
        $ch = curl_init();
        curl_setopt($ch, CURLOPT_URL, self::$baseURL . '/pay-systems?token=' . $this->config->get('betatransfer_token'));
        curl_setopt($ch, CURLOPT_POST, 1);
        curl_setopt($ch, CURLOPT_POSTFIELDS, http_build_query([
            'currency' => $currency,
            'plugin' => 'opencart',
        ]));
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
        $response = curl_exec($ch);
        curl_close($ch);

        return json_decode($response, true);
    }

    protected function generateSign($params)
    {
        if (!empty($params['sign'])) {
            unset($params['sign']);
        }

        array_push($params, $this->config->get('betatransfer_secret'));
        return md5(implode('', $params));
    }

    protected function sendRequest($params)
    {
        $curl = curl_init();
        curl_setopt_array($curl, array(
            CURLOPT_URL => self::$baseURL . '/payment?token=' . $this->config->get('betatransfer_token'),
            CURLOPT_POST => 1,
            CURLOPT_RETURNTRANSFER => true,
            CURLOPT_POSTFIELDS => http_build_query($params),
        ));

        $response = curl_exec($curl);
        curl_close($curl);

        return json_decode($response, true);
    }

    protected function is_valid_payment_system($systems)
    {
        $arr = array_map(function($el) { return $el['name']; }, $systems);
        $ps = $this->config->get('betatransfer_payment_system');
        return !$ps || in_array($ps, $arr);
    }
}
